"""This module provides progress tracking utilities, including a custom rate column for progress bars and a function to track the progress of iterating over sequences."""

import lazy_loader as lazy

__getattr__, __dir__, __all__ = lazy.attach_stub(__name__, __file__)
del lazy
